//
//  UserSettings.swift
//  SMHSSchedule
//
//  Created by Jevon Mao on 4/28/21.
//

import Combine
import Foundation

final class UserSettings: ObservableObject {
    @Published(key: "developerSettings") var developerSettings = DeveloperSettings()
    @Published(key: "userSettings") var editableSettings = [EditableSetting]()
    @Published(key: "preferLegacySchedule") var preferLegacySchedule = false
    
    init(){
        if editableSettings.isEmpty {
            resetEditableSettings()
        }
        #if DEBUG
        #else
        resetDeveloperSettings()
        #endif

    }
    func resetEditableSettings()
    {
        let periods = 1...7
        var settings = [EditableSetting]()
        for period in periods {
            settings.append(.init(periodNumber: period, textContent: ""))
        }
        self.editableSettings = settings
    }
    
    func resetDeveloperSettings()
    {
        self.developerSettings = DeveloperSettings()
    }
}
struct EditableSetting: Codable, Hashable {
    var periodNumber: Int
    var title: String {
        "Period \(periodNumber)"
    }
    var textContent: String
    
    static let sampleSetting = EditableSetting(periodNumber: 1, textContent: "")
}
struct DeveloperSettings: Codable {
    var alwaysLoadingState: Bool = false
    var alwaysShowOnboarding: Bool = false
    var shouldCacheData: Bool = true
}
